@@ -88,7 +88,6 @@ class Resources(object):
     _job_event = r'job_events/\d+/'
     _job_event_children = r'job_events/\d+/children/'
     _job_events = 'job_events/'
-    _job_extra_credentials = _job + 'extra_credentials/'
     _job_host_summaries = r'jobs/\d+/job_host_summaries/'
     _job_host_summary = r'job_host_summaries/\d+/'
     _job_job_event = r'jobs/\d+/job_events/\d+/'
@@ -105,7 +104,6 @@ class Resources(object):
     _job_template_access_list = r'job_templates/\d+/access_list/'
     _job_template_callback = r'job_templates/\d+/callback/'
     _job_template_copy = r'job_templates/\d+/copy/'
-    _job_template_extra_credentials = _job_template + 'extra_credentials/'
     _job_template_jobs = r'job_templates/\d+/jobs/'
     _job_template_labels = r'job_templates/\d+/labels/'
     _job_template_launch = r'job_templates/\d+/launch/'
